package com.myoa.engineering.crawl.shopping.crawlhandler.parser;

import com.myoa.engineering.crawl.shopping.domain.entity.v2.Article;
import com.myoa.engineering.crawl.shopping.support.dto.constant.PpomppuBoardName;
import com.myoa.engineering.crawl.shopping.util.DateTimeUtils;
import com.myoa.engineering.crawl.shopping.util.NumberUtils;
import io.micrometer.core.instrument.util.StringUtils;
import lombok.extern.slf4j.Slf4j;
import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.jsoup.nodes.Element;
import org.jsoup.select.Elements;
import org.springframework.stereotype.Component;

import java.time.ZoneId;
import java.time.ZonedDateTime;
import java.time.format.DateTimeFormatter;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

@Slf4j
@Component
public final class PpomppuArticleParserV2 {

    private static final DateTimeFormatter DATE_TIME_FORMATTER = DateTimeFormatter.ofPattern("yy.MM.dd HH:mm:ss")
                                                                                  .withZone(ZoneId.of("Asia/Seoul"));

    private PpomppuArticleParserV2() {
    }

    public List<Article> parse(String html) {
        Elements trElements = converHtmlToTrElements(html);
        return trElements.stream()
                         .filter(this::isRealArticle)
                         .map(this::parse)
                         .toList();

    }

    private Elements converHtmlToTrElements(String data) {
        Document document = Jsoup.parse(data);
        Elements trList = document.getElementById("revolution_main_table").getElementsByTag("tr");
        return trList;
    }

    private boolean isRealArticle(Element tr) {
        Elements tdList = tr.getElementsByTag("td");
        if (tdList.size() != 6) {
            return false;
        }

        if (!hasOnlyNumeric(tdList.get(0))) {
            return false;
        }

        return true;
    }

    private Pattern pattern_numeric = Pattern.compile("\\d+");

    private boolean hasOnlyNumeric(Element td) {
        return pattern_numeric.matcher(td.text()).matches();
    }

    private Article parse(Element tr) {
        Elements tdList = tr.getElementsByTag("td");
        Long articleId = Long.parseLong(tdList.get(0).text());

        String title = tdList.get(2).text();
        String articleUrl = parseArticleUrl(tdList.get(2).getElementsByTag("a").attr("href"));
        String boardName = parseBoardName(title);
        Integer recommended = parseRecommended(tdList.get(4));
        Integer hit = NumberUtils.parseInt(tdList.get(5).text(), 0);
        ZonedDateTime registeredAt = DateTimeUtils.parse(tdList.get(3).text());

        return Article.builder()
                      .articleId(articleId)
                      .title(title)
                      .boardName(boardName)
                      .articleUrl(articleUrl)
                      .recommended(recommended)
                      .hit(hit)
                      .registeredAt(registeredAt)
                      .build();
    }

    public Integer parseRecommended(Element td) {
        final String voteString = td.text();
        if (StringUtils.isEmpty(voteString)) {
            return null;
        }

        final int voteUp = Integer.parseInt(td.text().split(" - ")[0]);
        final int voteDown = Integer.parseInt(td.text().split(" - ")[1]);
        int recommended = voteUp - voteDown;
        return recommended;
    }

    public static String parseArticleUrl(String data) {
        return PpomppuBoardName.ofViewPageUrl(data);
    }

    Pattern patternBoardName = Pattern.compile("\\[(.+?)\\]");

    public String parseBoardName(String fullTitle) {
        Matcher matcher = patternBoardName.matcher(fullTitle);
        String lastMatched = null;
        while (matcher.find()) {
            lastMatched = matcher.group(1);
        }
        return lastMatched;
    }

}
