package com.myoa.engineering.crawl.shopping.event.handler;

import com.myoa.engineering.crawl.shopping.domain.model.UserNotifyModel;
import com.myoa.engineering.crawl.shopping.domain.model.v2.AppUserModel;
import com.myoa.engineering.crawl.shopping.domain.model.v2.ArticleModel;
import com.myoa.engineering.crawl.shopping.domain.model.v2.SubscribedKeywordAggregatedModel;
import com.myoa.engineering.crawl.shopping.event.ArticleUpsertEvent;
import com.myoa.engineering.crawl.shopping.service.AppUserQueryService;
import com.myoa.engineering.crawl.shopping.service.SubscribedKeywordCacheService;
import com.myoa.engineering.crawl.shopping.service.UserNotifyService;
import com.myoa.engineering.crawl.shopping.support.dto.constant.CrawlTarget;
import org.springframework.context.event.EventListener;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

@Component
public class ArticleUpsertEventListener {

    private final SubscribedKeywordCacheService subscribedKeywordCacheService;
    private final AppUserQueryService appUserQueryService;
    private final UserNotifyService userNotifyService;


    public ArticleUpsertEventListener(SubscribedKeywordCacheService subscribedKeywordCacheService,
                                      AppUserQueryService appUserQueryService, UserNotifyService userNotifyService) {
        this.subscribedKeywordCacheService = subscribedKeywordCacheService;
        this.appUserQueryService = appUserQueryService;
        this.userNotifyService = userNotifyService;
    }


    @EventListener
    public void handleArticleUpsertEvent(ArticleUpsertEvent event) {
        Map<CrawlTarget, List<ArticleModel>> articleMap =
                ((List<ArticleModel>) event.getSource()).stream()
                                                        .collect(Collectors.groupingBy(ArticleModel::getCrawlTarget));
        List<AppUserModel> appUsers = appUserQueryService.findAll();

        appUsers.stream()
                .filter(AppUserModel::getEnabled)
                .map(user -> {
                    List<ArticleModel> filteredArticles = handleAhoCorasick(articleMap)
                            .apply(subscribedKeywordCacheService.getSubscribedKeywordsCached(user.getName()));
                    return UserNotifyModel.of(user.getName(), filteredArticles);
                })
                .forEach(this::notifyMessage);

    }

    private Function<Map<CrawlTarget, SubscribedKeywordAggregatedModel>, List<ArticleModel>> handleAhoCorasick(
            Map<CrawlTarget, List<ArticleModel>> articleMap) {
        return userTrieModel -> {
            return userTrieModel
                    .entrySet()
                    .stream().filter(e -> articleMap.containsKey(e.getKey()))
                    .map((entry) -> filterAhocorasick(articleMap.get(entry.getKey()), entry.getValue()))
                    .flatMap(List::stream)
                    .toList();
//            return UserNotifyModel.of(userTrieModel.values().stream().findFirst().get().getUserId(),
//                                      filteredArticle);
        };
    }

    private List<ArticleModel> filterAhocorasick(List<ArticleModel> articles,
                                                 SubscribedKeywordAggregatedModel trieModel) {
        return articles.stream()
                       .filter(article -> !trieModel.getAhoCorasickTrie()
                                                    .parseText(article.getTitle())
                                                    .isEmpty())
                       .toList();
        //ArticleUpsertEventListener::printArticle
    }

    private void notifyMessage(UserNotifyModel article) {
        System.out.println("article = " + article);
        if (article.getArticles().isEmpty()){
            return;
        }
        userNotifyService.notify("안녕 " + article.getUserId() + "\n" + article.getArticles());
    }

}
