package com.myoa.engineering.crawl.shopping.service;

import com.myoa.engineering.crawl.shopping.domain.entity.v2.Article;
import com.myoa.engineering.crawl.shopping.domain.model.v2.ArticleModel;
import com.myoa.engineering.crawl.shopping.event.ArticleUpsertEvent;
import com.myoa.engineering.crawl.shopping.infra.repository.v2.ArticleRepository;
import javax.transaction.Transactional;
import org.springframework.context.ApplicationEventPublisher;
import org.springframework.stereotype.Service;

import java.util.Comparator;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

@Service
public class ArticleCommandService {

    private final ArticleRepository articleRepository;
    private final ApplicationEventPublisher applicationEventPublisher;

    public ArticleCommandService(ArticleRepository articleRepository,
                                 ApplicationEventPublisher applicationEventPublisher) {
        this.articleRepository = articleRepository;
        this.applicationEventPublisher = applicationEventPublisher;
    }


    @Transactional
    public void upsert(List<Article> articles) {
        Map<Long, Article> articleMap = articles.stream()
                                                .collect(Collectors.toMap(Article::getArticleId, e -> e));

        List<Article> saved = articleRepository.findByArticleIdIn(articleMap.keySet());
        List<Article> updated = saved.stream()
                                     .filter(e -> articleMap.containsKey(e.getArticleId()))
                                     .map(e -> e.update(articleMap.get(e.getArticleId())))
                                     .peek(e -> articleMap.remove(e.getArticleId()))
                                     .sorted(Comparator.comparing(Article::getArticleId))
                                     .toList();

        List<Article> newArticles = articleMap.values()
                                              .stream()
                                              .sorted(Comparator.comparing(Article::getArticleId))
                                              .toList();

        articleRepository.saveAll(updated);
        articleRepository.saveAll(newArticles);

        publishEvent(newArticles);
    }

    private void publishEvent(List<Article> articles) {
        List<ArticleModel> articleModels =
                articles.stream()
                        .map(transformer)
                        .toList();
        applicationEventPublisher.publishEvent(new ArticleUpsertEvent(articleModels));
    }

    @Transactional
    public void upsert(Article article) {
        Article saved = articleRepository.findByArticleId(article.getArticleId())
                                         .orElse(article);
        articleRepository.save(saved);
    }

    public static Function<Article, ArticleModel> transformer =
            article -> ArticleModel.builder()
                                   .id(article.getId())
                                   .articleId(article.getArticleId())
                                   .crawlTarget(article.getCrawlTarget())
                                   .boardName(article.getBoardName())
                                   .articleUrl(article.getArticleUrl())
                                   .title(article.getTitle())
                                   .hit(article.getHit())
                                   .recommended(article.getRecommended())
                                   .registeredAt(article.getRegisteredAt())
                                   .build();
}
